@@ -23,7 +23,7 @@ REGION_REPORT_COLUMNS = [
     "STOIIP_OIL",
     "ASSOCIATEDOIL_GAS",
     "STOIIP_TOTAL",
-    "WATER_TOTAL",
+    "WIIP_TOTAL",
     "GIIP_GAS",
     "ASSOCIATEDGAS_OIL",
     "GIIP_TOTAL",
